/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.risk.impl;

import com.github.liaochong.myexcel.core.DefaultStreamExcelBuilder;
import com.github.liaochong.myexcel.core.SaxExcelReader;
import com.yiring.app.constant.risk.TemplateList;
import com.yiring.app.domain.risk.RiskWarn;
import com.yiring.app.domain.risk.RiskWarnRepository;
import com.yiring.app.excel.risk.RiskWarnExcel;
import com.yiring.app.param.risk.RiskWarnAddParam;
import com.yiring.app.param.risk.RiskWarnConditionParam;
import com.yiring.app.param.risk.RiskWarnModifyParam;
import com.yiring.app.param.risk.ThresholdParam;
import com.yiring.app.service.risk.RiskWarnService;
import com.yiring.app.util.JpaUtil;
import com.yiring.app.vo.risk.RiskWarnDetailVo;
import com.yiring.app.vo.risk.RiskWarnVo;
import com.yiring.common.constant.RegEx;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.vo.PageVo;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

/**
 * @author tml
 * @version 1.0
 * @date 2022/5/23 17:57
 */
@Slf4j
@Service
@Transactional(rollbackFor = Exception.class)
public class RiskWarnServiceImpl implements RiskWarnService {

    @Resource
    private RiskWarnRepository riskWarnRepository;

    @Override
    public Result<String> addOne(RiskWarnAddParam param) {
        Result<String> result = checkThreshold(param.getThresholds());
        if (Status.OK.value() != result.getStatus()) {
            return result;
        }
        RiskWarn riskWarn = param.transform();
        riskWarnRepository.saveAndFlush(riskWarn);
        return Result.ok();
    }

    @Override
    public Result<String> modifyOne(RiskWarnModifyParam param) {
        Result<String> result = checkThreshold(param.getThresholds());
        if (Status.OK.value() != result.getStatus()) {
            return result;
        }
        Optional<RiskWarn> optional = riskWarnRepository.findById(param.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "修改的风险预警信息不存在");
        }
        RiskWarn riskWarn = optional.get();
        RiskWarn transform = param.transform();
        JpaUtil.copyNotNullProperties(transform, riskWarn);
        return Result.ok();
    }

    private Result<String> checkThreshold(List<ThresholdParam> thresholds) {
        boolean flag = thresholds.get(0).getMin().endsWith("%");
        for (ThresholdParam item : thresholds) {
            if (item.getMin().endsWith("%") != item.getMax().endsWith("%") || item.getMin().endsWith("%") != flag) {
                return Result.no(Status.EXPECTATION_FAILED, "阈值区间要么都为百分比,要么都不为百分比");
            }
            if (!item.getMin().matches(RegEx.DIGIT_DECIMALS) || !item.getMax().matches(RegEx.DIGIT_DECIMALS)) {
                return Result.no(Status.EXPECTATION_FAILED, "请输入正确格式的数字");
            }
        }
        return Result.ok();
    }

    @Override
    public Result<String> removeOne(IdParam idParam) {
        riskWarnRepository.deleteById(idParam.getId());
        return Result.ok();
    }

    @Override
    public Result<RiskWarnDetailVo> findOne(IdParam idParam) {
        Optional<RiskWarn> optional = riskWarnRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "查询的风险预警信息不存在");
        }
        RiskWarnDetailVo vo = RiskWarnDetailVo.transform(optional.get());
        return Result.ok(vo);
    }

    @Override
    public Result<PageVo<RiskWarnVo>> findList(RiskWarnConditionParam conditionParam, PageParam pageParam) {
        Pageable pageable = PageParam.toPageable(pageParam);
        Specification<RiskWarn> specification = getSpecification(conditionParam);
        Page<RiskWarn> page = riskWarnRepository.findAll(specification, pageable);
        List<RiskWarnVo> voList = page.get().map(RiskWarnVo::transform).collect(Collectors.toList());
        PageVo<RiskWarnVo> pageVo = PageVo.build(voList, voList.size());
        return Result.ok(pageVo);
    }

    @Override
    public void exportRiskWarn(RiskWarnConditionParam conditionParam, HttpServletResponse response) {
        Specification<RiskWarn> specification = getSpecification(conditionParam);
        List<RiskWarn> page = riskWarnRepository.findAll(specification);
        List<RiskWarnExcel> list = page.stream().map(RiskWarnExcel::transform).collect(Collectors.toList());
        try (
            DefaultStreamExcelBuilder<RiskWarnExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(RiskWarnExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(18)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(list);

            String fileName = URLEncoder.encode("风险预警信息表.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("风险预警信息表导出失败: " + e.getMessage());
        }
    }

    @Override
    public Result<String> importRiskWarn(MultipartFile file) {
        if (file.isEmpty()) {
            return Result.no(Status.EXPECTATION_FAILED, "请选择一个excel文件");
        }
        List<RiskWarnExcel> riskWarnExcels;
        try (InputStream inputStream = file.getInputStream()) {
            riskWarnExcels =
                SaxExcelReader
                    .of(RiskWarnExcel.class)
                    .rowFilter(row -> row.getRowNum() > 0)
                    .ignoreBlankRow()
                    .read(inputStream);
        } catch (IOException e) {
            log.info(e.getMessage());
            throw new RuntimeException("文件导入异常");
        }
        return null;
    }

    @Override
    public void exportTemplate(HttpServletResponse response) {
        List<RiskWarnExcel> list = TemplateList.getTemplateList();
        try (
            DefaultStreamExcelBuilder<RiskWarnExcel> streamExcelBuilder = DefaultStreamExcelBuilder
                .of(RiskWarnExcel.class)
                .threadPool(Executors.newFixedThreadPool(2))
                .rowHeight(14)
                .titleRowHeight(18)
                .style(
                    "cell->vertical-align:center;text-align:center",
                    "title->vertical-align:center;text-align:center;font-weight:bold;font-family:等线"
                )
                .start()
        ) {
            streamExcelBuilder.append(list);

            String fileName = URLEncoder.encode("风险预警模板.xlsx", StandardCharsets.UTF_8);
            response.setContentType("application/octet-stream");
            response.setHeader("Access-Control-Expose-Headers", "Content-Disposition");
            response.setHeader("Content-Disposition", "attachment;filename=" + fileName);
            OutputStream out = response.getOutputStream();

            Workbook workbook = streamExcelBuilder.fixedTitles().build();
            workbook.write(out);
            workbook.close();
            out.flush();
            out.close();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("风险预警模板导出失败: " + e.getMessage());
        }
    }

    @Override
    public Result<String> batchEnable(Set<Long> ids, boolean enable) {
        int num = riskWarnRepository.batchEnable(ids, enable, LocalDateTime.now());
        return Result.ok((enable ? "启用了" : "停用了") + num + "条风险预警信息");
    }

    private Specification<RiskWarn> getSpecification(RiskWarnConditionParam conditionParam) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            if (conditionParam.getWorkAreaId() != null) {
                list.add(criteriaBuilder.equal(root.get(RiskWarn.Fields.workArea), conditionParam.getWorkAreaId()));
            }
            if (conditionParam.getMonitoring() != null) {
                list.add(criteriaBuilder.equal(root.get(RiskWarn.Fields.monitoring), conditionParam.getMonitoring()));
            }
            Predicate[] array = list.toArray(new Predicate[0]);
            return criteriaBuilder.and(array);
        };
    }
}
